package org.jboss.as.quickstarts.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.annotation.Resource;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet("/HelloWorldMDBServletClient")
public class HelloWorldMDBServletClient extends HttpServlet {

	private static final long serialVersionUID = -1;

	private static final int MSG_COUNT = 5;

	@Resource(mappedName = "java:/ConnectionFactory")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = "java:/queue/HELLOWORLDMDBQueue")
	private Queue queue;

	@Resource(mappedName = "java:/topic/HELLOWORLDMDBTopic")
	private Topic topic;

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		resp.setContentType("text/html");
		PrintWriter out = resp.getWriter();
		Connection connection = null;
		out.write("<h1>QUEUE / TOPIC</h1>");
		try {
			Destination destination;
			if (req.getParameterMap().keySet().contains("topic")) {
				destination = topic;
			} else {
				destination = queue;
			}
			out.write("<p>Sending messages to <em>" + destination + "</em></p>");

			connection = connectionFactory.createConnection();
			Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			MessageProducer messageProducer = session.createProducer(destination);

			// settings
			messageProducer.setDeliveryMode(DeliveryMode.PERSISTENT);
			// PERSISTENT = This delivery mode instructs the JMS provider to log the message to stable storage as part of the client's send operation.
			// NON_PERSISTENT = This is the lowest-overhead delivery mode because it does not require that the message be logged to stable storage.
			messageProducer.setTimeToLive(10000); // miliseconds to live

			connection.start();

			out.write("<h2>Following messages will be send to the destination:</h2>");

			// send text message
			TextMessage message = session.createTextMessage();
			for (int i = 0; i < MSG_COUNT; i++) {
				message.setText("This is TEXT message " + (i + 1));
				messageProducer.send(message);
				out.write("Message (" + i + "): " + message.getText() + "</br>");
			}

			// send object message
			ObjectMessage objectMessage = session.createObjectMessage();
			for (int i = 0; i < MSG_COUNT; i++) {
				objectMessage.setObject(new MessageWrapper("This is OBJECT message " + (i + 1)));
				messageProducer.send(objectMessage);
				out.write("Message (" + i + "): " + objectMessage.getObject() + "</br>");
			}

			out.write("<p><i>Go to your JBoss Application Server console or Server log to see the result of messages processing</i></p>");

		} catch (JMSException e) {
			e.printStackTrace();
			out.write("<h2>A problem occurred during the delivery of this message</h2>");
			out.write("</br>");
			out.write("<p><i>Go your the JBoss Application Server console or Server log to see the error stack trace</i></p>");
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (JMSException e) {
					e.printStackTrace();
				}
			}
			if (out != null) {
				out.close();
			}
		}
	}

	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		doGet(req, resp);
	}
}
